@@ -1,6 +1,6 @@
+import copy
 import itertools
 import math
-from collections import deque
 from functools import reduce
 from typing import Optional, Union, List
 
@@ -37,10 +37,10 @@ class Rating:
                 return True
             else:
                 return False
-        elif isinstance(other, Union[list, tuple]):
+        elif isinstance(other, (list, tuple)):
             if len(other) == 2:
                 for value in other:
-                    if not isinstance(value, Union[int, float]):
+                    if not isinstance(value, (int, float)):
                         raise ValueError(
                             f"The {other.__class__.__name__} contains an "
                             f"element '{value}' of type '{value.__class__.__name__}'"
@@ -58,6 +58,114 @@ class Rating:
                 "You can only compare Rating objects with each other or a list of two floats."
             )
 
+    def __lt__(self, other):
+        if isinstance(other, Rating):
+            if ordinal(self) < ordinal(other):
+                return True
+            else:
+                return False
+        elif isinstance(other, (list, tuple)):
+            if len(other) == 2:
+                for value in other:
+                    if not isinstance(value, (int, float)):
+                        raise ValueError(
+                            f"The {other.__class__.__name__} contains an "
+                            f"element '{value}' of type '{value.__class__.__name__}'"
+                        )
+                if ordinal(self) < ordinal([other[0], other[1]]):
+                    return True
+                else:
+                    return False
+            else:
+                raise ValueError(
+                    f"The '{other.__class__.__name__}' object has more than two floats."
+                )
+        else:
+            raise ValueError(
+                "You can only compare Rating objects with each other or a list of two floats."
+            )
+
+    def __gt__(self, other):
+        if isinstance(other, Rating):
+            if ordinal(self) > ordinal(other):
+                return True
+            else:
+                return False
+        elif isinstance(other, (list, tuple)):
+            if len(other) == 2:
+                for value in other:
+                    if not isinstance(value, (int, float)):
+                        raise ValueError(
+                            f"The {other.__class__.__name__} contains an "
+                            f"element '{value}' of type '{value.__class__.__name__}'"
+                        )
+                if ordinal(self) > ordinal([other[0], other[1]]):
+                    return True
+                else:
+                    return False
+            else:
+                raise ValueError(
+                    f"The '{other.__class__.__name__}' object has more than two floats."
+                )
+        else:
+            raise ValueError(
+                "You can only compare Rating objects with each other or a list of two floats."
+            )
+
+    def __le__(self, other):
+        if isinstance(other, Rating):
+            if ordinal(self) <= ordinal(other):
+                return True
+            else:
+                return False
+        elif isinstance(other, (list, tuple)):
+            if len(other) == 2:
+                for value in other:
+                    if not isinstance(value, (int, float)):
+                        raise ValueError(
+                            f"The {other.__class__.__name__} contains an "
+                            f"element '{value}' of type '{value.__class__.__name__}'"
+                        )
+                if ordinal(self) <= ordinal([other[0], other[1]]):
+                    return True
+                else:
+                    return False
+            else:
+                raise ValueError(
+                    f"The '{other.__class__.__name__}' object has more than two floats."
+                )
+        else:
+            raise ValueError(
+                "You can only compare Rating objects with each other or a list of two floats."
+            )
+
+    def __ge__(self, other):
+        if isinstance(other, Rating):
+            if ordinal(self) >= ordinal(other):
+                return True
+            else:
+                return False
+        elif isinstance(other, (list, tuple)):
+            if len(other) == 2:
+                for value in other:
+                    if not isinstance(value, (int, float)):
+                        raise ValueError(
+                            f"The {other.__class__.__name__} contains an "
+                            f"element '{value}' of type '{value.__class__.__name__}'"
+                        )
+                if ordinal(self) >= ordinal([other[0], other[1]]):
+                    return True
+                else:
+                    return False
+            else:
+                raise ValueError(
+                    f"The '{other.__class__.__name__}' object has more than two floats."
+                )
+        else:
+            raise ValueError(
+                "You can only compare Rating objects with each other or a list of two floats."
+            )
+
 
 def ordinal(agent: Union[Rating, list, tuple], **options) -> float:
     """
@@ -68,10 +176,10 @@ def ordinal(agent: Union[Rating, list, tuple], **options) -> float:
     :param options: Pass in a set of custom values for constants defined in the Weng-Lin paper.
     :return: A :class:`~float` object that represents a 1 dimensional value for a rating.
     """
-    if isinstance(agent, Union[list, tuple]):
+    if isinstance(agent, (list, tuple)):
         if len(agent) == 2:
             for value in agent:
-                if not isinstance(value, Union[int, float]):
+                if not isinstance(value, (int, float)):
                     raise ValueError(
                         f"The {agent.__class__.__name__} contains an "
                         f"element '{value}' of type '{value.__class__.__name__}'"
@@ -103,7 +211,7 @@ def create_rating(rating_list: List[Union[int, float]]) -> Rating:
         raise TypeError("Argument is already a 'Rating' object.")
     elif len(rating_list) == 2:
         for value in rating_list:
-            if not isinstance(value, Union[int, float]):
+            if not isinstance(value, (int, float)):
                 raise ValueError(
                     f"The {rating_list.__class__.__name__} contains an "
                     f"element '{value}' of type '{value.__class__.__name__}'"
@@ -143,9 +251,23 @@ def rate(teams: List[List[Rating]], **options) -> List[List[Rating]]:
     :param teams: A list of teams, where teams are lists of :class:`~openskill.rate.Rating` objects.
     :param rank: A list of :class:`~int` where the lower values represent the winners.
     :param score: A list of :class:`~int` where higher values represent the winners.
+    :param tau: A :class:`~float` that modifies the additive dynamics factor.
+    :param prevent_sigma_increase: A :class:`~bool` that prevents sigma from ever increasing.
     :param options: Pass in a set of custom values for constants defined in the Weng-Lin paper.
     :return: Returns a list of :class:`~openskill.rate.Rating` objects.
+
+    .. note::
+       ``tau`` will default to ``25/300`` in the next major version update.
     """
+    original_teams = copy.deepcopy(teams)
+    if "tau" in options:
+        tau_squared = options["tau"] * options["tau"]
+        for team_index, team in enumerate(teams):
+            for player_index, player in enumerate(team):
+                teams[team_index][player_index].sigma = math.sqrt(
+                    player.sigma * player.sigma + tau_squared
+                )
+
     if "rank" in options:
         rank = options["rank"]
     else:
@@ -167,31 +289,45 @@ def rate(teams: List[List[Rating]], **options) -> List[List[Rating]]:
     else:
         model = PlackettLuce(teams, team_rating=team_rating, **options)
 
+    processed_result = []
     if rank and tenet:
         result = model.calculate()
         result, old_tenet = unwind(tenet, result)
-        final_result = []
         for item in result:
             team = []
             for player in item:
                 team.append(create_rating(player))
-            final_result.append(team)
-        return final_result
+            processed_result.append(team)
     else:
         result = model.calculate()
-        final_result = []
         for item in result:
             team = []
             for player in item:
                 team.append(create_rating(player))
-            final_result.append(team)
-        return final_result
+            processed_result.append(team)
+
+    final_result = processed_result
+
+    if options.get("tau"):
+        if options.get("prevent_sigma_increase"):
+            final_result = []
+            for team_index, team in enumerate(processed_result):
+                final_team = []
+                for player_index, player in enumerate(team):
+                    player_original = original_teams[team_index][player_index]
+                    if player.sigma <= player_original.sigma:
+                        sigma = player.sigma
+                    else:
+                        sigma = player_original.sigma
+                    final_team.append(Rating(mu=player.mu, sigma=sigma))
+                final_result.append(final_team)
+    return final_result
 
 
 def predict_win(teams: List[List[Rating]], **options) -> List[Union[int, float]]:
     """
     Predict how likely a match up against teams of one or more agents will go.
-    This algorithm has a time complexity of O(n!) where 'n' is the number of teams.
+    This algorithm has a time complexity of O(n!/(n - 2)!) where 'n' is the number of teams.
 
     :param teams: A list of two or more teams, where teams are lists of :class:`~openskill.rate.Rating` objects.
     :return: A list of probabilities of each team winning.
@@ -200,9 +336,10 @@ def predict_win(teams: List[List[Rating]], **options) -> List[Union[int, float]]
         raise ValueError(f"Expected at least two teams.")
 
     n = len(teams)
+    denom = (n * (n - 1)) / 2
 
     pairwise_probabilities = []
-    for pairwise_subset in itertools.permutations(teams, len(teams)):
+    for pairwise_subset in itertools.permutations(teams, 2):
         current_team_a_rating = team_rating([pairwise_subset[0]])
         current_team_b_rating = team_rating([pairwise_subset[1]])
         mu_a = current_team_a_rating[0][0]
@@ -216,28 +353,18 @@ def predict_win(teams: List[List[Rating]], **options) -> List[Union[int, float]]
             )
         )
 
-    if n > 2:
-        cache = deque(pairwise_probabilities)
-        probabilities = []
-        partial = len(pairwise_probabilities) / n
-        while len(cache) > 0:
-            aggregate = []
-            for length in range(int(partial)):
-                aggregate.append(cache.popleft())
-            aggregate_sum = sum(aggregate)
-            aggregate_multiple = n
-            for length in range(1, n - 2):
-                aggregate_multiple *= n - length
-            probabilities.append(aggregate_sum / aggregate_multiple)
-        return probabilities
-    else:
-        return pairwise_probabilities
+    return [
+        (sum(team_prob) / denom)
+        for team_prob in itertools.zip_longest(
+            *[iter(pairwise_probabilities)] * (n - 1)
+        )
+    ]
 
 
 def predict_draw(teams: List[List[Rating]], **options) -> Union[int, float]:
     """
     Predict how likely a match up against teams of one or more agents will draw.
-    This algorithm has a time complexity of O(n!) where 'n' is the number of teams.
+    This algorithm has a time complexity of O(n!/(n - 2)!) where 'n' is the number of teams.
 
     :param teams: A list of two or more teams, where teams are lists of :class:`~openskill.rate.Rating` objects.
     :return: A :class:`~float` that represents the probability of the teams drawings.
@@ -255,7 +382,7 @@ def predict_draw(teams: List[List[Rating]], **options) -> Union[int, float]:
     )
 
     pairwise_probabilities = []
-    for pairwise_subset in itertools.permutations(teams, len(teams)):
+    for pairwise_subset in itertools.permutations(teams, 2):
         current_team_a_rating = team_rating([pairwise_subset[0]])
         current_team_b_rating = team_rating([pairwise_subset[1]])
         mu_a = current_team_a_rating[0][0]
@@ -273,10 +400,8 @@ def predict_draw(teams: List[List[Rating]], **options) -> Union[int, float]:
             )
         )
 
+    denom = 1
     if n > 2:
-        aggregate_multiple = n
-        for length in range(1, n):
-            aggregate_multiple *= n - length
-        return abs(sum(pairwise_probabilities)) / aggregate_multiple
-    else:
-        return abs(sum(pairwise_probabilities))
+        denom = n * (n - 1)
+
+    return abs(sum(pairwise_probabilities)) / denom
